<#
* Copyright(c) 2006-2018 ThreatSTOP, Inc. All Rights Reserved
* NOTICE: All information contained herein is, and remains the property of ThreatSTOP, Inc. and its suppliers, if any. The intellectual and
* technical concepts contained herein are proprietary to ThreatSTOP, Inc. and its suppliers and certain aspects thereof are protected by United
* States Patent No. 8,533,822 and United States Patent No. 8,869,237, and are protected by trade secret or copyright law.
* US Government Entities: The ThreatSTOP service, software and documentation, as applicable, are "commercial computer software" and
* "commercial computer software documentation" developed exclusively at private expense by Threatstop, Inc. ("Threatstop"). Pursuant to FAR
* 12.212 or DFARS 227 7202 and their successors, as applicable, use, reproduction and disclosure of the software and documentation is
* governed by the terms of Threatstop's commercial agreement.
#>
# ErrID start: 40
#
# 1. upload DNS log using TS API
# 2. Move uploaded log to Archive folder
# 3. Move failed logs to Failed folder after 24 hours

#Load config values into hash table:
$scriptPath = split-path -parent $MyInvocation.MyCommand.Definition
$iniPath = (get-item $scriptPath).parent.fullname
. "$scriptPath\functions.ps1"
Get-Content "$iniPath\ThreatSTOP.ini" | foreach-object -begin {$h=@{}} -process { $k = [regex]::split($_,'='); if(($k[0].CompareTo("") -ne 0) -and ($k[0].StartsWith("[") -ne $True -and $k[0].StartsWith("#")  -ne $True)) { $h.Add($k[0].Trim(), $k[1].Trim()) } }
$LogFolder = $h.LogFolder
$LogArchiveFolder = $h.LogArchiveFolder
$LogFailedFolder="$LogArchiveFolder\Failed"
$limit = (Get-Date).AddDays(0-$h.LogRetention)
$IP = $h.MyPublicIP
$TDID = $h.TDID
$proxy = ""
$proxy_credential = ""
if (-Not [string]::IsNullOrWhiteSpace($h.Proxy)) {
    $proxy = $h.Proxy
}
if (-Not [string]::IsNullOrWhiteSpace($h.ProxyCredential)) {
    $proxy_credential = $h.ProxyCredential
}
$url  = 'https://logs.threatstop.com/cgi-bin/logupload.pl'
$response=""
#create Log Archive folder if missing
if (!(Test-Path $LogArchiveFolder))
{
    Write-Host 'Creating Sent Directory'
    New-Item -ItemType Directory -Force -Path $LogArchiveFolder
}

#delete old files
Write-Host -ForegroundColor CYAN "Purging files older than $h.LogRetention days..."
Get-ChildItem -Path $LogArchiveFolder -Recurse | Where-Object { !$_.PSIsContainer -and $_.CreationTime -lt $limit } | Remove-Item -Force

#Get files to upload
$files = Get-ChildItem $LogFolder -Filter DNSPolicy*.log
for ($i=0; $i -lt $files.Count; $i++)
{
    $filepath = $files[$i].FullName
    $filename = $files[$i].Name
    $filesize = $files[$i].Length

    $md5 = Get-FileHash -Path $filepath -Algorithm MD5
    # echo "params: -FilePath $filepath -RESTURL $url -filename $filename -IP $IP -md5 $md5.Hash.ToLower() -filesize $filesize -TDID $TDID"
    $response = $FALSE
    if(-Not [string]::IsNullOrWhiteSpace($TDID)) {
        Write-Host -ForegroundColor GREEN "using identifier TDID: $TDID"
        $response = Upload-Log -FilePath $filepath -RESTURL $url -filename $filename -IP $IP -md5 $md5.Hash.ToLower() -filesize $filesize -TDID $TDID -Proxy $proxy -ProxyCredential $proxy_credential
    }
    else {
        Write-Host -ForegroundColor RED "using identifier IP: $IP"
        $response = Upload-Log -FilePath $filepath -RESTURL $url -filename $filename -IP $IP -md5 $md5.Hash.ToLower() -filesize $filesize -Proxy $proxy -ProxyCredential $proxy_credential
    }
    #Write-Host $response
    #check success
    if ($response -Match "MD5: OK")
    {
        Move-Item $filepath "$LogArchiveFolder\$filename"
    }
    else
    {
        Write-EventLog -LogName Application -Source 'ThreatSTOP' -EntryType Error -EventId 41 -Message $response
        #if failed file is more than 24 hours old, move to .\Failed
        if ((Get-Item $FilePath).CreationTime -lt (Get-Date).AddDays(-1))
        {

            if(!(Test-Path $LogFailedFolder))
            {
                New-Item -Path $LogFailedFolder -ItemType Directory
            }
            Move-Item $FilePath "$LogFailedFolder\$filename" -Force
        }
    }
    Start-Sleep -s 2
}
if ($i -gt 0)
{
    $msg = "Uploaded $i DNS log(s)"
    Write-EventLog -LogName Application -Source 'ThreatSTOP' -EntryType Information -EventId 40 -Message $msg
}
#must set compression each time for new files:
Invoke-WmiMethod -Path "Win32_Directory.Name='$LogArchiveFolder'" -Name compress
if(Test-Path $LogFailedFolder){Invoke-WmiMethod -Path "Win32_Directory.Name='$LogFailedFolder'" -Name compress}